/*
    American option pricer under proportional transaction costs
    Copyright (C) 2011 Alet Roux alet.roux@york.ac.uk

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef PAYOFFPUTCASHASK_H
#define PAYOFFPUTCASHASK_H

////#include <ostream>

#include "singlestrike.h"

///Put option with cash delivery based on ask price.
/**
 Pays strike() minus ask price of share in cash (and no shares).
 @author Alet Roux <alet.roux@york.ac.uk>
 */
class PayoffPutCashAsk : public PayoffSingleStrike
{
public:
	PayoffPutCashAsk (const coefficient strike)
			: PayoffSingleStrike (strike)
	{}

	Portfolio operator() (const Spot& spot) const
	{
		coefficient payoff = strike()-spot.ask();
		return greaterthan (payoff, 0.0) ? Portfolio (0, payoff) : Portfolio (0, 0);
	}

	PayoffPutCashAsk* clone() const
	{
		return new PayoffPutCashAsk (*this);
	}

};

std::ostream& operator<< (std::ostream& output, const PayoffPutCashAsk& put);


#endif // PAYOFFPUTCASHASK_H
