/*
    American option pricer under proportional transaction costs
    Copyright (C) 2011 Alet Roux alet.roux@york.ac.uk

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef STATISTICSGATHERER_ALL_H
#define STATISTICSGATHERER_ALL_H

//#include <vector>
//#include <ostream>

///Collects all objects of interest.
template<class T>
		class StatisticsGathererAll : public StatisticsGatherer<T>
{
	public:
	/**
	 * Constructor.
	 */
		StatisticsGathererAll ()
		{}

		operator StatisticsGathererInitial<T>() const
		{
			StatisticsGathererInitial<T> gatherer;
			gatherer.dump_result (0, 0, _object[0][0]);

			return gatherer;
		}

		StatisticsGathererPath<T> operator() (const std::vector<size_t>& path) const
		{
			StatisticsGathererPath<T> gatherer (path);
			for (size_t t = 0; t < _object.size(); t++)
				for (size_t k = 0; k < _object[t].size(); k++)
					gatherer.dump_result (t,k,_object[t][k]);
			return gatherer;
		};

		StatisticsGathererSome<T> operator() (const std::vector< std::vector<size_t> >& path) const
		{
			StatisticsGathererSome<T> gatherer (path);
			for (size_t t = 0; t < _object.size(); t++)
				for (size_t k = 0; k < _object[t].size(); k++)
					gatherer.dump_result (t,k,_object[t][k]);
			return gatherer;
		};

		virtual void dump_result (const size_t t, const size_t k, const T& object)
		{
			if (t >= _object.size())
				_object.resize(t+1);
			if (k >= _object[t].size())
				_object[t].resize(k+1);

			_object[t][k] = object;
		}

		std::vector<T> operator[] (const size_t t) const
		{
			return _object[t];
		};

		std::vector<T>& operator[] (const size_t t)
		{
			return _object[t];
		};

		T operator() (const size_t t, const size_t k) const
		{
			return _object[t][k];
		};

		T& operator() (const size_t t, const size_t k)
		{
			return _object[t][k];
		};

	///Number of steps
		size_t size() const
		{
			return _object.size();
		}

	///Number of steps
		size_t size (const size_t t) const
		{
			return _object[t].size();
		}

	private:
		std::vector< std::vector<T> > _object;
};

template<class T>
		inline std::ostream& operator<< (std::ostream& output, const StatisticsGathererAll<T>& function)
{
	for (size_t t = 0; t < function.size(); t++)
		for (size_t k = 0; k < function.size (t); k++)
			output << "(" << t << "," << k << "): " << function (t,k) << std::endl;
	return output;
}


#endif // STATISTICSGATHERER_ALL_H
